=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2010-08-20 18:37:11 +0000
+++ drizzled/drizzled.cc	2010-08-24 01:36:17 +0000
@@ -939,7 +939,7 @@
     global_system_variables.storage_engine= engine;
   }
 
-  if (plugin::XaResourceManager::recoverAllXids(0))
+  if (plugin::XaResourceManager::recoverAllXids())
   {
     unireg_abort(1);
   }

=== modified file 'drizzled/module/loader.cc'
--- drizzled/module/loader.cc	2010-08-06 17:57:15 +0000
+++ drizzled/module/loader.cc	2010-08-20 19:08:50 +0000
@@ -27,7 +27,6 @@
 #include <boost/program_options.hpp>
 
 #include "drizzled/option.h"
-#include "drizzled/my_hash.h"
 #include "drizzled/internal/m_string.h"
 
 #include "drizzled/plugin.h"
@@ -95,7 +94,6 @@
 */
 static memory::Root plugin_mem_root(4096);
 static uint32_t global_variables_dynamic_size= 0;
-static HASH bookmark_hash;
 
 
 /*
@@ -107,26 +105,24 @@
   Item *item;
 };
 
-
-/*
-  stored in bookmark_hash, this structure is never removed from the
-  hash and is used to mark a single offset for a session local variable
-  even if plugins have been uninstalled and reinstalled, repeatedly.
-  This structure is allocated from plugin_mem_root.
-
-  The key format is as follows:
-    1 byte         - variable type code
-    name_len bytes - variable name
-    '\0'           - end of key
-*/
-struct st_bookmark
+class Bookmark
 {
-  uint32_t name_len;
+public:
+  Bookmark() :
+    type_code(0),
+    offset(0),
+    version(0),
+    key("")
+  {}
+  uint8_t type_code;
   int offset;
   uint32_t version;
-  char key[1];
+  string key;
 };
 
+typedef boost::unordered_map<string, Bookmark> bookmark_unordered_map;
+static bookmark_unordered_map bookmark_hash;
+
 
 /*
   sys_var class for access to all plugin variables visible to the user
@@ -363,15 +359,6 @@
 }
 
 
-static unsigned char *get_bookmark_hash_key(const unsigned char *buff,
-                                            size_t *length, bool)
-{
-  struct st_bookmark *var= (st_bookmark *)buff;
-  *length= var->name_len + 1;
-  return (unsigned char*) var->key;
-}
-
-
 /*
   The logic is that we first load and initialize all compiled in plugins.
   From there we load up the dynamic types (assuming we have not been told to
@@ -388,14 +375,6 @@
   if (initialized)
     return false;
 
-  if (hash_init(&bookmark_hash, &my_charset_bin, 16, 0, 0,
-                  get_bookmark_hash_key, NULL, HASH_UNIQUE))
-  {
-    tmp_root.free_root(MYF(0));
-    return true;
-  }
-
-
   initialized= 1;
 
   vector<string> builtin_list;
@@ -567,8 +546,6 @@
   }
 
   /* Dispose of the memory */
-
-  hash_free(&bookmark_hash);
   plugin_mem_root.free_root(MYF(0));
 
   global_variables_dynamic_size= 0;
@@ -805,11 +782,9 @@
   return(var);
 }
 
-static const string make_bookmark_name(const string &plugin, const char *name, int flags)
+static const string make_bookmark_name(const string &plugin, const char *name)
 {
-  /* Embed the flags into the first char of the string */
-  string varname(1, static_cast<char>(flags & PLUGIN_VAR_TYPEMASK));
-  varname.append(plugin);
+  string varname(plugin);
   varname.push_back('_');
   varname.append(name);
 
@@ -828,20 +803,19 @@
   Returns the 'bookmark' for the named variable.
   LOCK_system_variables_hash should be at least read locked
 */
-static st_bookmark *find_bookmark(const string &plugin, const char *name, int flags)
+static Bookmark *find_bookmark(const string &plugin, const char *name, int flags)
 {
-  st_bookmark *result= NULL;
-
   if (!(flags & PLUGIN_VAR_SessionLOCAL))
     return NULL;
 
-  const string varname(make_bookmark_name(plugin, name, flags));
-
-
-  result= (st_bookmark*) hash_search(&bookmark_hash,
-                                     (const unsigned char*) varname.c_str(), varname.size() - 1);
-
-  return result;
+  const string varname(make_bookmark_name(plugin, name));
+
+  bookmark_unordered_map::iterator iter= bookmark_hash.find(varname);
+  if (iter != bookmark_hash.end())
+  {
+    return &((*iter).second);
+  }
+  return NULL;
 }
 
 
@@ -850,14 +824,14 @@
   returns null for non session-local variables.
   Requires that a write lock is obtained on LOCK_system_variables_hash
 */
-static st_bookmark *register_var(const string &plugin, const char *name,
+static Bookmark *register_var(const string &plugin, const char *name,
                                  int flags)
 {
   if (!(flags & PLUGIN_VAR_SessionLOCAL))
     return NULL;
 
   uint32_t size= 0, offset, new_size;
-  st_bookmark *result;
+  Bookmark *result= NULL;
 
   switch (flags & PLUGIN_VAR_TYPEMASK) {
   case PLUGIN_VAR_BOOL:
@@ -883,19 +857,17 @@
 
   if (!(result= find_bookmark(plugin, name, flags)))
   {
-    const string varname(make_bookmark_name(plugin, name, flags));
+    const string varname(make_bookmark_name(plugin, name));
 
-    result= static_cast<st_bookmark*>(plugin_mem_root.alloc_root(sizeof(struct st_bookmark) + varname.size() + 1));
-    memset(result->key, 0, varname.size()+1);
-    memcpy(result->key, varname.c_str(), varname.size());
-    result->name_len= varname.size() - 2;
-    result->offset= -1;
+    Bookmark new_bookmark;
+    new_bookmark.key= varname;
+    new_bookmark.offset= -1;
 
     assert(size && !(size & (size-1))); /* must be power of 2 */
 
     offset= global_system_variables.dynamic_variables_size;
     offset= (offset + size - 1) & ~(size - 1);
-    result->offset= (int) offset;
+    new_bookmark.offset= (int) offset;
 
     new_size= (offset + size + 63) & ~63;
 
@@ -935,14 +907,12 @@
     global_system_variables.dynamic_variables_version++;
     max_system_variables.dynamic_variables_version++;
 
-    result->version= global_system_variables.dynamic_variables_version;
+    new_bookmark.version= global_system_variables.dynamic_variables_version;
+    new_bookmark.type_code= flags;
 
     /* this should succeed because we have already checked if a dup exists */
-    if (my_hash_insert(&bookmark_hash, (unsigned char*) result))
-    {
-      fprintf(stderr, "failed to add placeholder to hash");
-      assert(0);
-    }
+    bookmark_hash.insert(make_pair(varname, new_bookmark));
+    result= find_bookmark(plugin, name, flags);
   }
   return result;
 }
@@ -968,8 +938,6 @@
   if (!session->variables.dynamic_variables_ptr ||
       (uint32_t)offset > session->variables.dynamic_variables_head)
   {
-    uint32_t idx;
-
     char *tmpptr= NULL;
     if (!(tmpptr= (char *)realloc(session->variables.dynamic_variables_ptr,
                                   global_variables_dynamic_size)))
@@ -992,16 +960,18 @@
       now we need to iterate through any newly copied 'defaults'
       and if it is a string type with MEMALLOC flag, we need to strdup
     */
-    for (idx= 0; idx < bookmark_hash.records; idx++)
+    bookmark_unordered_map::iterator iter= bookmark_hash.begin();
+    for (; iter != bookmark_hash.end() ; ++iter)
     {
       sys_var_pluginvar *pi;
       sys_var *var;
-      st_bookmark *v= (st_bookmark*) hash_element(&bookmark_hash,idx);
+      const Bookmark &v= (*iter).second;
+      const string var_name((*iter).first);
 
-      if (v->version <= session->variables.dynamic_variables_version ||
-          !(var= intern_find_sys_var(v->key + 1, v->name_len, true)) ||
+      if (v.version <= session->variables.dynamic_variables_version ||
+          !(var= intern_find_sys_var(var_name.c_str(), var_name.size(), true)) ||
           !(pi= var->cast_pluginvar()) ||
-          v->key[0] != (pi->plugin_var->flags & PLUGIN_VAR_TYPEMASK))
+          v.type_code != (pi->plugin_var->flags & PLUGIN_VAR_TYPEMASK))
         continue;
 
       /* Here we do anything special that may be required of the data types */
@@ -1091,18 +1061,19 @@
 */
 static void cleanup_variables(Session *session, struct system_variables *vars)
 {
-  st_bookmark *v;
   sys_var_pluginvar *pivar;
   sys_var *var;
   int flags;
 
-  for (uint32_t idx= 0; idx < bookmark_hash.records; idx++)
+  bookmark_unordered_map::iterator iter= bookmark_hash.begin();
+  for (; iter != bookmark_hash.end() ; ++iter)
   {
-    v= (st_bookmark*) hash_element(&bookmark_hash, idx);
-    if (v->version > vars->dynamic_variables_version ||
-        !(var= intern_find_sys_var(v->key + 1, v->name_len, true)) ||
+    const Bookmark &v= (*iter).second;
+    const string key_name((*iter).first);
+    if (v.version > vars->dynamic_variables_version ||
+        !(var= intern_find_sys_var(key_name.c_str(), key_name.size(), true)) ||
         !(pivar= var->cast_pluginvar()) ||
-        v->key[0] != (pivar->plugin_var->flags & PLUGIN_VAR_TYPEMASK))
+        v.type_code != (pivar->plugin_var->flags & PLUGIN_VAR_TYPEMASK))
       continue;
 
     flags= pivar->plugin_var->flags;
@@ -1439,7 +1410,7 @@
   char *optname, *p;
   int index= 0, offset= 0;
   drizzle_sys_var *opt, **plugin_option;
-  st_bookmark *v;
+  Bookmark *v;
 
   string name(plugin_name);
   transform(name.begin(), name.end(), name.begin(), ::tolower);
@@ -1569,7 +1540,7 @@
       if (opt->flags & PLUGIN_VAR_NOCMDOPT)
         continue;
 
-      optname= (char*) mem_root->memdup_root(v->key + 1, (optnamelen= v->name_len) + 1);
+      optname= (char*) mem_root->memdup_root(v->key.c_str(), (optnamelen= v->key.size()) + 1);
     }
 
     /* convert '_' to '-' */
@@ -1656,7 +1627,7 @@
   option *opts= NULL;
   int error;
   drizzle_sys_var *o;
-  struct st_bookmark *var;
+  Bookmark *var;
   uint32_t len, count= EXTRA_OPTIONS;
 
   if (test_module->getManifest().init_options != NULL)
@@ -1721,7 +1692,7 @@
 
       if ((var= find_bookmark(test_module->getName(), o->name, o->flags)))
       {
-        v= new sys_var_pluginvar(var->key + 1, o);
+        v= new sys_var_pluginvar(var->key.c_str(), o);
       }
       else
       {

=== modified file 'drizzled/plugin/storage_engine.cc'
--- drizzled/plugin/storage_engine.cc	2010-08-05 17:51:54 +0000
+++ drizzled/plugin/storage_engine.cc	2010-08-20 20:54:13 +0000
@@ -32,7 +32,6 @@
 #include <google/protobuf/io/zero_copy_stream.h>
 #include <google/protobuf/io/zero_copy_stream_impl.h>
 
-#include "drizzled/my_hash.h"
 #include "drizzled/cached_directory.h"
 
 #include <drizzled/definitions.h>

=== modified file 'drizzled/plugin/xa_resource_manager.cc'
--- drizzled/plugin/xa_resource_manager.cc	2010-04-16 05:36:53 +0000
+++ drizzled/plugin/xa_resource_manager.cc	2010-08-20 20:54:13 +0000
@@ -20,7 +20,6 @@
 
 #include "config.h"
 
-#include "drizzled/my_hash.h"
 #include "drizzled/cached_directory.h"
 
 #include <drizzled/definitions.h>
@@ -69,26 +68,28 @@
   @note
     there are three modes of operation:
     - automatic recover after a crash
-    in this case commit_list != 0, tc_heuristic_recover==0
+    in this case commit_list.size() != 0, tc_heuristic_recover==0
     all xids from commit_list are committed, others are rolled back
     - manual (heuristic) recover
-    in this case commit_list==0, tc_heuristic_recover != 0
+    in this case commit_list.size()==0, tc_heuristic_recover != 0
     DBA has explicitly specified that all prepared transactions should
     be committed (or rolled back).
-    - no recovery (MySQL did not detect a crash)
-    in this case commit_list==0, tc_heuristic_recover == 0
+    - no recovery (Drizzle did not detect a crash)
+    in this case commit_list.size()==0, tc_heuristic_recover == 0
     there should be no prepared transactions in this case.
 */
 class XaRecover : unary_function<XaResourceManager *, void>
 {
+private:
   int trans_len, found_foreign_xids, found_my_xids;
   bool result;
   XID *trans_list;
-  HASH *commit_list;
+  const XaResourceManager::commit_list_set &commit_list;
   bool dry_run;
 public:
   XaRecover(XID *trans_list_arg, int trans_len_arg,
-            HASH *commit_list_arg, bool dry_run_arg) 
+            const XaResourceManager::commit_list_set& commit_list_arg,
+            bool dry_run_arg) 
     : trans_len(trans_len_arg), found_foreign_xids(0), found_my_xids(0),
       result(false),
       trans_list(trans_list_arg), commit_list(commit_list_arg),
@@ -130,8 +131,8 @@
           continue;
         }
         // recovery mode
-        if (commit_list ?
-            hash_search(commit_list, (unsigned char *)&x, sizeof(x)) != 0 :
+        if (commit_list.size() ?
+            commit_list.find(x) != commit_list.end() :
             tc_heuristic_recover == TC_HEURISTIC_RECOVER_COMMIT)
         {
           resource_manager->xaCommitXid(trans_list+i);
@@ -147,15 +148,21 @@
   }
 };
 
-int XaResourceManager::recoverAllXids(HASH *commit_list)
+int XaResourceManager::recoverAllXids()
+{
+  const XaResourceManager::commit_list_set empty_commit_set;
+  return recoverAllXids(empty_commit_set);
+}
+
+int XaResourceManager::recoverAllXids(const XaResourceManager::commit_list_set &commit_list)
 {
   XID *trans_list= NULL;
   int trans_len= 0;
 
-  bool dry_run= (commit_list==0 && tc_heuristic_recover==0);
+  bool dry_run= (commit_list.size() == 0 && tc_heuristic_recover==0);
 
   /* commit_list and tc_heuristic_recover cannot be set both */
-  assert(commit_list==0 || tc_heuristic_recover==0);
+  assert(commit_list.size() == 0 || tc_heuristic_recover == 0);
 
   if (xa_resource_managers.size() <= 1)
     return 0;
@@ -173,7 +180,7 @@
     return(1);
   }
 
-  if (commit_list)
+  if (commit_list.size())
     errmsg_printf(ERRMSG_LVL_INFO, _("Starting crash recovery..."));
 
   XaRecover recover_func(trans_list, trans_len, commit_list, dry_run);
@@ -198,7 +205,7 @@
                     recover_func.getMyXIDs(), opt_tc_log_file);
     return(1);
   }
-  if (commit_list)
+  if (commit_list.size())
     errmsg_printf(ERRMSG_LVL_INFO, _("Crash recovery finished."));
   return(0);
 }

=== modified file 'drizzled/plugin/xa_resource_manager.h'
--- drizzled/plugin/xa_resource_manager.h	2010-03-03 20:21:57 +0000
+++ drizzled/plugin/xa_resource_manager.h	2010-08-20 20:54:13 +0000
@@ -21,6 +21,8 @@
 #ifndef DRIZZLED_PLUGIN_XA_RESOURCE_MANAGER_H
 #define DRIZZLED_PLUGIN_XA_RESOURCE_MANAGER_H
 
+#include <boost/unordered_set.hpp>
+
 namespace drizzled
 {
 
@@ -69,12 +71,14 @@
     return doXaRecover(append_to, len);
   }
 
+  typedef ::boost::unordered_set<my_xid> commit_list_set;
   /** 
    * The below static class methods wrap the interaction
    * of the vector of registered XA storage engines.
    */
   static int commitOrRollbackXID(XID *xid, bool commit);
-  static int recoverAllXids(HASH *commit_list);
+  static int recoverAllXids();
+  static int recoverAllXids(const commit_list_set& commit_list);
 
   /* Class Methods for operating on plugin */
   static bool addPlugin(plugin::XaResourceManager *manager);

=== modified file 'drizzled/plugin/xa_storage_engine.cc'
--- drizzled/plugin/xa_storage_engine.cc	2010-04-16 05:36:53 +0000
+++ drizzled/plugin/xa_storage_engine.cc	2010-08-20 20:54:13 +0000
@@ -20,7 +20,6 @@
 
 #include "config.h"
 
-#include "drizzled/my_hash.h"
 #include "drizzled/cached_directory.h"
 
 #include <drizzled/definitions.h>

=== modified file 'plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/alter_table.result'
--- plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/alter_table.result	2010-08-20 16:56:14 +0000
+++ plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/alter_table.result	2010-08-22 00:02:17 +0000
@@ -176,7 +176,7 @@
 alter table t2 rename t1, add c char(10) comment "no comment";
 show columns from t1;
 Field	Type	Null	Default	Default_is_NULL	On_Update
-i	INTEGER	NO		NO
+i	INTEGER	NO		NO	
 c	VARCHAR	YES		YES	
 drop table t1;
 create table t1 (a int, b int);
@@ -527,13 +527,13 @@
 t1	CREATE TABLE `t1` (
   `a` int DEFAULT NULL,
   KEY `i1` (`a`) USING BTREE
-) ENGINE=DEFAULT COLLATE = utf8_general_ci ROW_FORMAT='COMPACT'
+) ENGINE=DEFAULT ROW_FORMAT='COMPACT' COLLATE = utf8_general_ci
 DROP INDEX i1 ON t1;
 SHOW CREATE TABLE t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
   `a` int DEFAULT NULL
-) ENGINE=DEFAULT COLLATE = utf8_general_ci ROW_FORMAT='COMPACT'
+) ENGINE=DEFAULT ROW_FORMAT='COMPACT' COLLATE = utf8_general_ci
 DROP TABLE t1;
 DROP TABLE IF EXISTS bug24219;
 DROP TABLE IF EXISTS bug24219_2;

=== modified file 'plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/auto_increment.result'
--- plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/auto_increment.result	2010-08-20 15:01:12 +0000
+++ plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/auto_increment.result	2010-08-22 00:02:17 +0000
@@ -159,7 +159,7 @@
   `a` int NOT NULL AUTO_INCREMENT,
   `b` int DEFAULT NULL,
   PRIMARY KEY (`a`) USING BTREE
-) ENGINE=InnoDB COLLATE = utf8_general_ci 
+) ENGINE=InnoDB COLLATE = utf8_general_ci
 update t1 set a=0 where b=5;
 select * from t1 order by b;
 a	b
@@ -312,7 +312,7 @@
   `t1_id` int NOT NULL AUTO_INCREMENT,
   PRIMARY KEY (`t1_id`) USING BTREE,
   KEY `t1_name` (`t1_name`(191)) USING BTREE
-) ENGINE=InnoDB COLLATE = utf8_general_ci AUTO_INCREMENT=1000 
+) ENGINE=InnoDB COLLATE = utf8_general_ci AUTO_INCREMENT=1000
 DROP TABLE `t1`;
 create table t1(a int not null auto_increment primary key);
 create table t2(a int not null auto_increment primary key, t1a int);

=== modified file 'plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/basic_innodb_table_proto_table.result'
--- plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/basic_innodb_table_proto_table.result	2010-08-20 16:56:14 +0000
+++ plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/basic_innodb_table_proto_table.result	2010-08-22 00:02:17 +0000
@@ -4,7 +4,7 @@
   `table_name` varbinary(192),
   `message` blob,
   PRIMARY KEY (`table_name`) USING BTREE
-) ENGINE=InnoDB COLLATE = binary 
+) ENGINE=InnoDB COLLATE = binary
 select * from data_dictionary.innodb_table_definitions;
 table_name	message
 use data_dictionary;

=== modified file 'plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/type_blob.result'
--- plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/type_blob.result	2010-08-20 15:01:12 +0000
+++ plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/type_blob.result	2010-08-22 00:02:17 +0000
@@ -651,7 +651,7 @@
 show create table t1;
 Table	Create Table
 t1	CREATE TABLE `t1` (
-  `a` text
+  `a` text DEFAULT ''
 ) ENGINE=DEFAULT COLLATE = utf8_general_ci
 insert into t1 values (default);
 select * from t1;

